@@ -1,5 +1,3 @@
-from __future__ import annotations
-
 from copy import deepcopy
 from dataclasses import dataclass, field
 from enum import Enum
@@ -33,12 +31,11 @@ class EndpointCollection:
     """ A bunch of endpoints grouped under a tag that will become a module """
 
     tag: str
-    endpoints: List[Endpoint] = field(default_factory=list)
-    relative_imports: Set[str] = field(default_factory=set)
+    endpoints: List["Endpoint"] = field(default_factory=list)
     parse_errors: List[ParseError] = field(default_factory=list)
 
     @staticmethod
-    def from_data(*, data: Dict[str, oai.PathItem]) -> Dict[str, EndpointCollection]:
+    def from_data(*, data: Dict[str, oai.PathItem]) -> Dict[str, "EndpointCollection"]:
         """ Parse the openapi paths data to get EndpointCollections by tag """
         endpoints_by_tag: Dict[str, EndpointCollection] = {}
 
@@ -58,13 +55,24 @@ class EndpointCollection:
                     )
                     collection.parse_errors.append(endpoint)
                     continue
-
+                for error in endpoint.errors:
+                    error.header = f"WARNING parsing {method.upper()} {path} within {tag}."
+                    collection.parse_errors.append(error)
                 collection.endpoints.append(endpoint)
-                collection.relative_imports.update(endpoint.relative_imports)
 
         return endpoints_by_tag
 
 
+def generate_operation_id(*, path: str, method: str) -> str:
+    """ Generate an operationId from a path """
+    clean_path = path.replace("{", "").replace("}", "").replace("/", "_")
+    if clean_path.startswith("_"):
+        clean_path = clean_path[1:]
+    if clean_path.endswith("_"):
+        clean_path = clean_path[:-1]
+    return f"{method}_{clean_path}"
+
+
 @dataclass
 class Endpoint:
     """
@@ -85,6 +93,7 @@ class Endpoint:
     form_body_reference: Optional[Reference] = None
     json_body: Optional[Property] = None
     multipart_body_reference: Optional[Reference] = None
+    errors: List[ParseError] = field(default_factory=list)
 
     @staticmethod
     def parse_request_form_body(body: oai.RequestBody) -> Optional[Reference]:
@@ -114,7 +123,7 @@ class Endpoint:
         return None
 
     @staticmethod
-    def _add_body(endpoint: Endpoint, data: oai.Operation) -> Union[ParseError, Endpoint]:
+    def _add_body(endpoint: "Endpoint", data: oai.Operation) -> Union[ParseError, "Endpoint"]:
         """ Adds form or JSON body to Endpoint if included in data """
         endpoint = deepcopy(endpoint)
         if data.requestBody is None or isinstance(data.requestBody, oai.Reference):
@@ -128,30 +137,41 @@ class Endpoint:
         endpoint.multipart_body_reference = Endpoint.parse_multipart_body(data.requestBody)
 
         if endpoint.form_body_reference:
-            endpoint.relative_imports.add(import_string_from_reference(endpoint.form_body_reference, prefix="..models"))
+            endpoint.relative_imports.add(
+                import_string_from_reference(endpoint.form_body_reference, prefix="...models")
+            )
         if endpoint.multipart_body_reference:
             endpoint.relative_imports.add(
-                import_string_from_reference(endpoint.multipart_body_reference, prefix="..models")
+                import_string_from_reference(endpoint.multipart_body_reference, prefix="...models")
             )
         if json_body is not None:
             endpoint.json_body = json_body
-            endpoint.relative_imports.update(endpoint.json_body.get_imports(prefix="..models"))
+            endpoint.relative_imports.update(endpoint.json_body.get_imports(prefix="..."))
         return endpoint
 
     @staticmethod
-    def _add_responses(endpoint: Endpoint, data: oai.Responses) -> Union[Endpoint, ParseError]:
+    def _add_responses(endpoint: "Endpoint", data: oai.Responses) -> "Endpoint":
         endpoint = deepcopy(endpoint)
         for code, response_data in data.items():
             response = response_from_data(status_code=int(code), data=response_data)
             if isinstance(response, ParseError):
-                return ParseError(detail=f"cannot parse response of endpoint {endpoint.name}", data=response.data)
+                endpoint.errors.append(
+                    ParseError(
+                        detail=(
+                            f"Cannot parse response for status code {code}, "
+                            f"response will be ommitted from generated client"
+                        ),
+                        data=response.data,
+                    )
+                )
+                continue
             if isinstance(response, (RefResponse, ListRefResponse)):
-                endpoint.relative_imports.add(import_string_from_reference(response.reference, prefix="..models"))
+                endpoint.relative_imports.add(import_string_from_reference(response.reference, prefix="...models"))
             endpoint.responses.append(response)
         return endpoint
 
     @staticmethod
-    def _add_parameters(endpoint: Endpoint, data: oai.Operation) -> Union[Endpoint, ParseError]:
+    def _add_parameters(endpoint: "Endpoint", data: oai.Operation) -> Union["Endpoint", ParseError]:
         endpoint = deepcopy(endpoint)
         if data.parameters is None:
             return endpoint
@@ -161,7 +181,7 @@ class Endpoint:
             prop = property_from_data(name=param.name, required=param.required, data=param.param_schema)
             if isinstance(prop, ParseError):
                 return ParseError(detail=f"cannot parse parameter of endpoint {endpoint.name}", data=prop.data)
-            endpoint.relative_imports.update(prop.get_imports(prefix="..models"))
+            endpoint.relative_imports.update(prop.get_imports(prefix="..."))
 
             if param.param_in == ParameterLocation.QUERY:
                 endpoint.query_parameters.append(prop)
@@ -174,17 +194,19 @@ class Endpoint:
         return endpoint
 
     @staticmethod
-    def from_data(*, data: oai.Operation, path: str, method: str, tag: str) -> Union[Endpoint, ParseError]:
+    def from_data(*, data: oai.Operation, path: str, method: str, tag: str) -> Union["Endpoint", ParseError]:
         """ Construct an endpoint from the OpenAPI data """
 
         if data.operationId is None:
-            return ParseError(data=data, detail="Path operations with operationId are not yet supported")
+            name = generate_operation_id(path=path, method=method)
+        else:
+            name = data.operationId
 
         endpoint = Endpoint(
             path=path,
             method=method,
             description=utils.remove_string_escapes(data.description) if data.description else "",
-            name=data.operationId,
+            name=name,
             requires_security=bool(data.security),
             tag=tag,
         )
@@ -193,8 +215,6 @@ class Endpoint:
         if isinstance(result, ParseError):
             return result
         result = Endpoint._add_responses(result, data.responses)
-        if isinstance(result, ParseError):
-            return result
         result = Endpoint._add_body(result, data)
 
         return result
@@ -215,8 +235,8 @@ class Model:
     relative_imports: Set[str]
 
     @staticmethod
-    def from_data(*, data: oai.Schema, name: str) -> Union[Model, ParseError]:
-        """ A single Model from its OAI data
+    def from_data(*, data: oai.Schema, name: str) -> Union["Model", ParseError]:
+        """A single Model from its OAI data
 
         Args:
             data: Data of a single Schema
@@ -239,7 +259,7 @@ class Model:
                 required_properties.append(p)
             else:
                 optional_properties.append(p)
-            relative_imports.update(p.get_imports(prefix=""))
+            relative_imports.update(p.get_imports(prefix=".."))
 
         model = Model(
             reference=ref,
@@ -259,7 +279,7 @@ class Schemas:
     errors: List[ParseError] = field(default_factory=list)
 
     @staticmethod
-    def build(*, schemas: Dict[str, Union[oai.Reference, oai.Schema]]) -> Schemas:
+    def build(*, schemas: Dict[str, Union[oai.Reference, oai.Schema]]) -> "Schemas":
         """ Get a list of Schemas from an OpenAPI dict """
         result = Schemas()
         for name, data in schemas.items():
@@ -273,6 +293,7 @@ class Schemas:
                     required=True,
                     default=data.default,
                     values=EnumProperty.values_from_list(data.enum),
+                    nullable=data.nullable,
                 )
                 continue
             s = Model.from_data(data=data, name=name)
@@ -295,7 +316,7 @@ class GeneratorData:
     enums: Dict[str, EnumProperty]
 
     @staticmethod
-    def from_dict(d: Dict[str, Dict[str, Any]]) -> Union[GeneratorData, GeneratorError]:
+    def from_dict(d: Dict[str, Dict[str, Any]]) -> Union["GeneratorData", GeneratorError]:
         """ Create an OpenAPI from dict """
         try:
             openapi = oai.OpenAPI.parse_obj(d)
