@@ -1,9 +1,8 @@
-from __future__ import annotations
-
 from dataclasses import InitVar, dataclass, field
-from datetime import date, datetime
 from itertools import chain
-from typing import Any, ClassVar, Dict, Generic, List, Optional, Set, TypeVar, Union
+from typing import Any, ClassVar, Dict, Generic, List, Optional, Set, Type, TypeVar, Union
+
+from dateutil.parser import isoparse
 
 from .. import schema as oai
 from .. import utils
@@ -61,7 +60,8 @@ class Property:
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         if self.nullable or not self.required:
             return {"from typing import Optional"}
@@ -92,7 +92,7 @@ class StringProperty(Property):
     _type_string: ClassVar[str] = "str"
 
     def _validate_default(self, default: Any) -> str:
-        return f'"{utils.remove_string_escapes(default)}"'
+        return f"{utils.remove_string_escapes(default)!r}"
 
 
 @dataclass
@@ -109,19 +109,19 @@ class DateTimeProperty(Property):
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         imports = super().get_imports(prefix=prefix)
-        imports.update({"import datetime", "from typing import cast"})
+        imports.update({"import datetime", "from typing import cast", "from dateutil.parser import isoparse"})
         return imports
 
     def _validate_default(self, default: Any) -> str:
-        for format_string in ("%Y-%m-%dT%H:%M:%S", "%Y-%m-%dT%H:%M:%S%z"):
-            try:
-                return repr(datetime.strptime(default, format_string))
-            except (TypeError, ValueError):
-                continue
-        raise ValidationError
+        try:
+            isoparse(default)
+        except (TypeError, ValueError) as e:
+            raise ValidationError from e
+        return f"isoparse({default!r})"
 
 
 @dataclass
@@ -136,17 +136,19 @@ class DateProperty(Property):
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         imports = super().get_imports(prefix=prefix)
-        imports.update({"import datetime", "from typing import cast"})
+        imports.update({"import datetime", "from typing import cast", "from dateutil.parser import isoparse"})
         return imports
 
     def _validate_default(self, default: Any) -> str:
         try:
-            return repr(date.fromisoformat(default))
+            isoparse(default).date()
         except (TypeError, ValueError) as e:
             raise ValidationError() from e
+        return f"isoparse({default!r}).date()"
 
 
 @dataclass
@@ -161,10 +163,11 @@ class FileProperty(Property):
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         imports = super().get_imports(prefix=prefix)
-        imports.update({f"from {prefix}.types import File", "from dataclasses import astuple"})
+        imports.update({f"from {prefix}types import File"})
         return imports
 
 
@@ -228,25 +231,16 @@ class ListProperty(Property, Generic[InnerProp]):
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         imports = super().get_imports(prefix=prefix)
         imports.update(self.inner_property.get_imports(prefix=prefix))
         imports.add("from typing import List")
-        if self.default is not None:
-            imports.add("from dataclasses import field")
-            imports.add("from typing import cast")
         return imports
 
-    def _validate_default(self, default: Any) -> str:
-        if not isinstance(default, list):
-            raise ValidationError()
-
-        default = list(map(self.inner_property._validate_default, default))
-        if isinstance(self.inner_property, RefProperty):  # Fix enums to use the actual value
-            default = str(default).replace("'", "")
-
-        return f"field(default_factory=lambda: cast({self.get_type_string()}, {default}))"
+    def _validate_default(self, default: Any) -> None:
+        return None
 
 
 @dataclass
@@ -269,7 +263,8 @@ class UnionProperty(Property):
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         imports = super().get_imports(prefix=prefix)
         for inner_prop in self.inner_properties:
@@ -287,16 +282,18 @@ class UnionProperty(Property):
         raise ValidationError()
 
 
-_existing_enums: Dict[str, EnumProperty] = {}
+_existing_enums: Dict[str, "EnumProperty"] = {}
+ValueType = Union[str, int]
 
 
 @dataclass
 class EnumProperty(Property):
     """ A property that should use an enum """
 
-    values: Dict[str, str]
+    values: Dict[str, ValueType]
     reference: Reference = field(init=False)
     title: InitVar[str]
+    value_type: Type[ValueType] = field(init=False)
 
     template: ClassVar[str] = "enum_property.pyi"
 
@@ -311,16 +308,21 @@ class EnumProperty(Property):
             reference = Reference.from_ref(f"{reference.class_name}{dedup_counter}")
 
         self.reference = reference
+
+        for value in self.values.values():
+            self.value_type = type(value)
+            break
+
         super().__post_init__()
         _existing_enums[self.reference.class_name] = self
 
     @staticmethod
-    def get_all_enums() -> Dict[str, EnumProperty]:
+    def get_all_enums() -> Dict[str, "EnumProperty"]:
         """ Get all the EnumProperties that have been registered keyed by class name """
         return _existing_enums
 
     @staticmethod
-    def get_enum(name: str) -> Optional[EnumProperty]:
+    def get_enum(name: str) -> Optional["EnumProperty"]:
         """ Get all the EnumProperties that have been registered keyed by class name """
         return _existing_enums.get(name)
 
@@ -336,27 +338,33 @@ class EnumProperty(Property):
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         imports = super().get_imports(prefix=prefix)
-        imports.add(f"from {prefix}.{self.reference.module_name} import {self.reference.class_name}")
+        imports.add(f"from {prefix}models.{self.reference.module_name} import {self.reference.class_name}")
         return imports
 
     @staticmethod
-    def values_from_list(values: List[str]) -> Dict[str, str]:
+    def values_from_list(values: List[ValueType]) -> Dict[str, ValueType]:
         """ Convert a list of values into dict of {name: value} """
-        output: Dict[str, str] = {}
+        output: Dict[str, ValueType] = {}
 
         for i, value in enumerate(values):
+            if isinstance(value, int):
+                if value < 0:
+                    output[f"VALUE_NEGATIVE_{-value}"] = value
+                else:
+                    output[f"VALUE_{value}"] = value
+                continue
             if value[0].isalpha():
                 key = value.upper()
             else:
                 key = f"VALUE_{i}"
             if key in output:
                 raise ValueError(f"Duplicate key {key} in Enum")
-            sanitized_key = utils.fix_keywords(utils.sanitize(key))
+            sanitized_key = utils.snake_case(key).upper()
             output[sanitized_key] = utils.remove_string_escapes(value)
-
         return output
 
     def _validate_default(self, default: Any) -> str:
@@ -391,12 +399,13 @@ class RefProperty(Property):
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         imports = super().get_imports(prefix=prefix)
         imports.update(
             {
-                f"from {prefix}.{self.reference.module_name} import {self.reference.class_name}",
+                f"from {prefix}models.{self.reference.module_name} import {self.reference.class_name}",
                 "from typing import Dict",
                 "from typing import cast",
             }
@@ -423,7 +432,8 @@ class DictProperty(Property):
         Get a set of import strings that should be included when this property is used somewhere
 
         Args:
-            prefix: A prefix to put before any relative (local) module names.
+            prefix: A prefix to put before any relative (local) module names. This should be the number of . to get
+            back to the root of the generated client.
         """
         imports = super().get_imports(prefix=prefix)
         imports.add("from typing import Dict")
@@ -432,10 +442,8 @@ class DictProperty(Property):
             imports.add("from typing import cast")
         return imports
 
-    def _validate_default(self, default: Any) -> str:
-        if isinstance(default, dict):
-            return repr(default)
-        raise ValidationError
+    def _validate_default(self, default: Any) -> None:
+        return None
 
 
 def _string_based_property(
@@ -444,14 +452,33 @@ def _string_based_property(
     """ Construct a Property from the type "string" """
     string_format = data.schema_format
     if string_format == "date-time":
-        return DateTimeProperty(name=name, required=required, default=data.default, nullable=data.nullable,)
+        return DateTimeProperty(
+            name=name,
+            required=required,
+            default=data.default,
+            nullable=data.nullable,
+        )
     elif string_format == "date":
-        return DateProperty(name=name, required=required, default=data.default, nullable=data.nullable,)
+        return DateProperty(
+            name=name,
+            required=required,
+            default=data.default,
+            nullable=data.nullable,
+        )
     elif string_format == "binary":
-        return FileProperty(name=name, required=required, default=data.default, nullable=data.nullable,)
+        return FileProperty(
+            name=name,
+            required=required,
+            default=data.default,
+            nullable=data.nullable,
+        )
     else:
         return StringProperty(
-            name=name, default=data.default, required=required, pattern=data.pattern, nullable=data.nullable,
+            name=name,
+            default=data.default,
+            required=required,
+            pattern=data.pattern,
+            nullable=data.nullable,
         )
 
 
@@ -462,7 +489,11 @@ def _property_from_data(
     name = utils.remove_string_escapes(name)
     if isinstance(data, oai.Reference):
         return RefProperty(
-            name=name, required=required, reference=Reference.from_ref(data.ref), default=None, nullable=False,
+            name=name,
+            required=required,
+            reference=Reference.from_ref(data.ref),
+            default=None,
+            nullable=False,
         )
     if data.enum:
         return EnumProperty(
@@ -481,18 +512,37 @@ def _property_from_data(
                 return PropertyError(detail=f"Invalid property in union {name}", data=sub_prop_data)
             sub_properties.append(sub_prop)
         return UnionProperty(
-            name=name, required=required, default=data.default, inner_properties=sub_properties, nullable=data.nullable,
+            name=name,
+            required=required,
+            default=data.default,
+            inner_properties=sub_properties,
+            nullable=data.nullable,
         )
     if not data.type:
         return PropertyError(data=data, detail="Schemas must either have one of enum, anyOf, or type defined.")
     if data.type == "string":
         return _string_based_property(name=name, required=required, data=data)
     elif data.type == "number":
-        return FloatProperty(name=name, default=data.default, required=required, nullable=data.nullable,)
+        return FloatProperty(
+            name=name,
+            default=data.default,
+            required=required,
+            nullable=data.nullable,
+        )
     elif data.type == "integer":
-        return IntProperty(name=name, default=data.default, required=required, nullable=data.nullable,)
+        return IntProperty(
+            name=name,
+            default=data.default,
+            required=required,
+            nullable=data.nullable,
+        )
     elif data.type == "boolean":
-        return BooleanProperty(name=name, required=required, default=data.default, nullable=data.nullable,)
+        return BooleanProperty(
+            name=name,
+            required=required,
+            default=data.default,
+            nullable=data.nullable,
+        )
     elif data.type == "array":
         if data.items is None:
             return PropertyError(data=data, detail="type array must have items defined")
@@ -500,10 +550,19 @@ def _property_from_data(
         if isinstance(inner_prop, PropertyError):
             return PropertyError(data=inner_prop.data, detail=f"invalid data in items of array {name}")
         return ListProperty(
-            name=name, required=required, default=data.default, inner_property=inner_prop, nullable=data.nullable,
+            name=name,
+            required=required,
+            default=data.default,
+            inner_property=inner_prop,
+            nullable=data.nullable,
         )
     elif data.type == "object":
-        return DictProperty(name=name, required=required, default=data.default, nullable=data.nullable,)
+        return DictProperty(
+            name=name,
+            required=required,
+            default=data.default,
+            nullable=data.nullable,
+        )
     return PropertyError(data=data, detail=f"unknown type {data.type}")
 
 
